#!/bin/sh
VERSION=8.4.0
PLATFORM=`uname`
    
current_dir=`pwd`
path=`dirname $0`
file=`basename $0`
if test "$path" != ""; then
    DIR=$path
else
    DIR=$current_dir
fi

echo "*********************************************************************"
echo "*            DB2 Alphablox Analytics v${VERSION} Setup                   *"
echo "*                                                                   *"
echo "* (c) Copyright International Business Machines Corporation 2006.   *"
echo "*                      All rights reserved.                         *" 
echo "* US Government Users Restricted Rights - Use, duplication or       *"
echo "* disclosure restricted by GSA ADP Schedule Contract with IBM Corp. *"
echo "*                                                                   *"
echo "*********************************************************************"
echo ""

echo "This script will setup your system to install DB2 Alphablox Analytics."
echo "It will expand the contents of the package and then prompt you to proceed"
echo "with the installation."
echo ""

ECHO_INTERPRET_ESCAPES="echo"
TAIL_SWITCH="-1"

if test "${PLATFORM}" = "SunOS"; then
    PLATFORM=Solaris
fi
if test "${PLATFORM}" = "Linux"; then
	PLATFORM=Linux
	ECHO_INTERPRET_ESCAPES="echo -e"
	TAIL_SWITCH="-n 1"
fi

echo "The graphical (AWT) installation program requires the X-Windows"
echo "libraries. By default, the installation will run in console mode which"
echo "allows you to run the installation from a command-line terminal with no"
echo "X-Windows libraries installed (for example, from a telnet session)."
echo ""
echo "Enter the number corresponding to the installation you want to run:"
if test "${PLATFORM}" != "AIX"; then
    echo "1. Run the default console installation"
    echo "2. Run the graphical installation"
else
    echo "1. Run console installation; GUI installation not available"
fi
$ECHO_INTERPRET_ESCAPES "[1]: \c"
read REPLY
case $REPLY in
    2) CONSOLE=0;;
    *) CONSOLE=1;;
esac

SPACEREQ=300000

while true
do
    echo ""
    echo "Enter the temporary directory for setup to expand the package: [/tmp]"
    read TEMP_DIR
    if test -z "${TEMP_DIR}"; then
        TEMP_DIR=/tmp
        echo "--> Temporary directory set to default of '/tmp'."
    fi
    
    tempval=${TEMP_DIR}
    tempval=`echo $tempval | sed 's/ //g'`  
    
    if test "$tempval" != "${TEMP_DIR}"; then
        echo "!!! Spaces are not allowed in the temporary directory path."
    else
        if test ! -d ${TEMP_DIR}; then
          mkdir -p ${TEMP_DIR} 
          TEMP_DIR_CREATED=Y 
        else
          TEMP_DIR_CREATED=N 
        fi
        echo ""

        if test "${PLATFORM}" != "AIX"; then
            SpaceAvail=`df -k ${TEMP_DIR} | tail $TAIL_SWITCH  | awk '{print $4}'`
        else
            SpaceAvail=`df -k ${TEMP_DIR} | tail $TAIL_SWITCH  | awk '{print $3}'`
        fi


        if test ${SpaceAvail} -lt ${SPACEREQ}; then
          echo "Not enough space available on ${TEMP_DIR}.  Space Available = ${SpaceAvail}, Space Required = ${SPACEREQ}"
          echo "Do you want to proceed the setup with this directory? (Y/N) [N]" 
          read response
          if test "${response}" != "y" -a "${response}" != "Y"; then 
            continue
          else
            break
          fi
        else
          break    
        fi 
    fi
done    

if test ! -d "${TEMP_DIR}"; then
    mkdir -p ${TEMP_DIR}
    TEMP_DIR_CREATED=Y
fi

IATEMPDIR=${TEMP_DIR}; export IATEMPDIR

echo ""
echo "Expanding the install package to $TEMP_DIR ..."
echo "This might take several minutes."
echo ""

if test ${PLATFORM} = "Linux"; then
    if test "$(uname -a | grep x86_64)X" != "X"; then
	cd ${DIR}/Linux/amd64
    elif test "$(uname -a | grep ppc)X" != "X"; then
	cd ${DIR}/Linux/ppc
    else	
    	cd ${DIR}/Linux
    fi
else
    cd ${DIR}/UNIX/${PLATFORM}
fi

if test $CONSOLE -eq 0; then
    sh install.bin -i gui
else
    sh install.bin -i console
fi

if test "$TEMP_DIR_CREATED" = "Y"; then
    rm -rf $TEMP_DIR
fi

exit 0

echo ""

