<%@ page contentType="text/html; charset=UTF-8" %>

<%@ taglib uri="bloxtld" prefix="blox" %>
<%@ taglib uri="bloxformtld" prefix="bloxform" %>

<blox:grid id="APGridBlox"
	visible="false"
    menubarVisible="false"
	width="700"
	height="150">
    <blox:toolbar visible="false" />
	<blox:data
		dataSourceName="QCC-Essbase"
		query='<SYM <ROW ("All Products") <ICHILD "All Products" <COLUMN ("All Time Periods", Scenario) <CHILD "All Time Periods" <CHILD Scenario Sales !'
		useAliases="true" />
</blox:grid>

<html>
<head>
	<title>Using Numeric Properties</title>
	<link rel="stylesheet" href="/BloxSampler/styles/example.css" type="text/css">
	<blox:header>
		<blox:clientBean name="APGridBlox">
			<blox:method name="setDefaultCellFormat" />
			<blox:method name="setCellFormat" />
			<blox:method name="setCellAlert" />
		</blox:clientBean>
	</blox:header>

<script language="JavaScript">

function showDefaultCellFormat() {
	if (CheckboxForm.DefaultCellFormatCheckbox.checked)
		document.APGridBlox.setDefaultCellFormat("$,##0");
	else
		document.APGridBlox.setDefaultCellFormat("");
}

function showCellFormat() {
  if (CheckboxForm.CellFormatCheckbox.checked)
    {
    document.APGridBlox.setCellFormat(1,"condition=any,format=$,##0.00,scope={Scenario:Forecast,Actual,Variance}");
	document.APGridBlox.setCellFormat(2,"condition=any,format=0.00'%',scope={Scenario:Variance %}");
	}
  else 
  {
    document.APGridBlox.setCellFormat(1,"");
		document.APGridBlox.setCellFormat(2,"");
  }	
}

function showCellAlert() {
  if (CheckboxForm.CellAlertCheckbox.checked) {
	document.APGridBlox.setCellAlert(1,"condition=lt,value=0,background=red,scope={Scenario:Variance %}");
	document.APGridBlox.setCellAlert(2,"condition=between,value=0,value2=10,background=yellow,scope={Scenario:Variance %}");
	document.APGridBlox.setCellAlert(3,"condition=gt,value=10,background=green,scope={Scenario:Variance %}");
	}
  else 
	document.APGridBlox.setCellAlert(1,"condition=lt,value=0,background=white,scope={Scenario:Variance %}");
	
}

</script>

</head>

<body>

<h2>Using Numeric Properties</h2>
  
<p>
Numbers should be formatted  so that users can more
  easily read and interpret their data. In this example, a couple of GridBlox
  numeric
  formatting
  properties
  are demonstrated to give you an idea of the possibilities as well as issues
  to consider when designing views. For complete details on number formatting,
  including more examples, refer to the GridBlox Reference section of the <em>Developer's
  Reference</em>.
</p>

<blox:display bloxRef="APGridBlox" />

<form name="CheckboxForm">
  <p> <b>
    <input type="checkbox" name="DefaultCellFormatCheckbox" value="checkbox" onClick="showDefaultCellFormat()">
    defaultCellFormat=&quot;$,##0&quot;</b><br>
    The defaultCellFormat property sets a global format for all of the data cells 
    in a view. Keep in mind that the defaultCellFormat property is global in scope, 
    and may introduce undesirable side effects. When you select the option here, 
    you should notice that not only do the Actual, Forecast, and Variance numbers 
    show a dollar sign ($) in front of the numbers, but the Variance % column 
    does, too. This would not make sense and would be unacceptable. <br>
  </p>
  <p> <b> 
    <input type="checkbox" name="CellFormatCheckbox" value="checkbox" onClick="showCellFormat()">
    &lt;blox:cellFormat index=&quot;1&quot;<br>
    &nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; condition=&quot;any&quot;<br>
    &nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;</b><b>format=&quot;$,##0.00&quot;&nbsp;<br>
    &nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; </b><b>scope=&quot;{Scenario:Forecast,Actual,Variance}&quot;</b>/&gt;<br>
    &nbsp; &nbsp; &nbsp;<b>&lt;blox:cellFormat index=&quot;2&quot;<br>
    &nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;condition=&quot;any&quot; <br>
    &nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; format=&quot;0.00"%&quot;<br>
    &nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; scope=&quot;{Scenario:Variance 
    %}&quot;/&gt;</b><br>
    To  fine-tune the appearances of numbers, you can use the cellFormat
     property tags to limit the effects to specific rows, columns, or other conditions.
    
    This example shows how cellFormat property tags can be used to display Actual,
     Forecast, and Variance with dollar signs, but display Variance % with a percent
    
    symbol at the end of each number.</p>
</form>
  
<p><b>Note:</b> The above
  examples actually use the Blox client API to dynamically demonstrate
  the effects of various 
  property settings. Instead of using Blox client API methods on your Blox, you
  should use the appropriate numeric properties and nested tags to define
  the data formatting.</p>

<hr>
<a href="../../viewSource.jsp?file=Essbase/Presenting/numericProperties.jsp">View Source</a>

</body>

</html>