<%@ page contentType="text/html; charset=UTF-8" %>

<%@ page import="com.alphablox.blox.uimodel.*,
                 com.alphablox.blox.uimodel.core.event.IEventHandler,
                 com.alphablox.blox.uimodel.core.event.ComponentRebuiltNotify,
                 java.util.ArrayList,
                 com.alphablox.blox.uimodel.core.tree.Tree,
                 com.alphablox.blox.uimodel.core.tree.TreeNode,
                 com.alphablox.blox.uimodel.core.*"%>

<%@ taglib uri="bloxtld" prefix="blox" %>

<blox:data id="dataBlox"
	dataSourceName="QCC-MSAS"
    query="SELECT {[All Time Periods].Children} ON COLUMNS, {[All Products].Children, [All Products]} ON ROWS FROM QCC WHERE ([Measures].[Sales])" />

<html>

<head>
<TITLE>Removing Other Axis on DataLayout</TITLE>
<blox:header />
<link rel="stylesheet" href="/BloxSampler/styles/example.css" type="text/css">
</head>

<body>
<h2>Removing Other Axis on DataLayout</h2>

<p>This example uses the Blox UI model to create a DataLayout filter that prevents the Other axis from being
available on the DataLayout panel and in the drop-down menus.</p>

<blox:present id="dlfPresentBloxMSAS"
	width="700"
 	height="500">

<%
    final PresentBloxModel model = dlfPresentBloxMSAS.getPresentBloxModel();

    model.getController().addEventHandler( new IEventHandler() {
        public boolean handleComponentRebuiltNotify( ComponentRebuiltNotify event ) throws Exception {
            if ( !(event.getComponent() instanceof DataLayoutBrixModel) )
                return false;

            DataLayoutBrixModel layout = (DataLayoutBrixModel)event.getComponent();

            // Hide the other axis container
            if (layout.searchForComponent( ModelConstants.DATA_LAYOUT_OTHER_CONTAINER ) != null) {

                layout.searchForComponent( ModelConstants.DATA_LAYOUT_OTHER_CONTAINER ).setVisible( false );

                // Remove the move to other option from all dimensions
                removeMoveToOther( layout.searchForComponent( ModelConstants.DATA_LAYOUT_ROW_CONTAINER ), false );
                removeMoveToOther( layout.searchForComponent( ModelConstants.DATA_LAYOUT_COLUMN_CONTAINER ), false );
                removeMoveToOther( layout.searchForComponent( ModelConstants.DATA_LAYOUT_PAGE_CONTAINER ), false );

            }
            else if (layout.searchForComponent( ModelConstants.DATA_LAYOUT_TREE ) != null) {

                Tree dataLayoutTree = (Tree)model.searchForComponent(ModelConstants.DATA_LAYOUT_TREE);
                TreeNode[] layoutNodes = dataLayoutTree.getNodes();

                for (int n = 0; n < layoutNodes.length; n++) {
                    if (layoutNodes[n].getTitle().toLowerCase().indexOf("other") != -1) {
                        dataLayoutTree.removeNode(layoutNodes[n]);
                        dataLayoutTree.changed();
                    }
                    else if (layoutNodes[n].hasChildren()) {
                        // Remove the move to other option from all dimensions
                        for (int z = 0; z < layoutNodes[n].size(); z++)
                            removeMoveToOther( layoutNodes[n].get(z).getRightClickMenu(), true );
                    }
                }
            }

            return false;
        }
    } );

%>

	<blox:data bloxRef="dataBlox" />

</blox:present>

<%!
    // Find the move to other option in the data layout panel and remove it
    public void removeMoveToOther( Component container, boolean isTree )
    {

        if (isTree && container != null && container instanceof Menu) {

            Menu menu = (Menu)container;

            for ( int j=0; j < menu.size(); j++ ) {
                if ( menu.get(j).getTitle().indexOf("Move to other") != -1 ) {
                    menu.remove( j );
                    break;
                }
            }

        }
        else if (!isTree) {

            ArrayList list = ((ComponentContainer)container).searchForAllComponents( DropDownList.class );

            for ( int i=0; i < list.size(); i++ ) {
                DropDownList ddl = (DropDownList)list.get( i );
                for ( int j=0; j < ddl.size(); j++ ) {
                    if ( "Move to other".equalsIgnoreCase( ddl.get(j) ) ) {
                        ddl.remove( j );
                        break;
                    }
                }
            }

        }

    }
%>

<hr>
<a href="../../viewSource.jsp?file=MSAS/UIExtensibility/removingOther.jsp">View Source</a>

</body>

</html>