package com.alphablox.bloxsampler.layout;

import com.alphablox.blox.uimodel.tags.layout.AbstractLayout;
import com.alphablox.blox.uimodel.core.grid.GridCell;
import com.alphablox.blox.uimodel.GridBrixCellModel;
import com.alphablox.blox.DataViewBlox;
import com.alphablox.blox.ServerBloxException;
import com.alphablox.blox.data.mdb.*;

public class CustomLayout extends AbstractLayout {
    protected String getFormatName() {
        return "Custom Layout (show MSAS member attributes)";
    }

    protected void layoutCell(GridCell gridCell, DataViewBlox dataViewBlox) throws Exception {

        // Make sure this is a header cell, and it belongs to the grid  brix (i.e. not added by another layout)
        if ( !gridCell.isColumnHeader() && !gridCell.isRowHeader() )
            return;

        Property[] properties = getHeaderCellProperties(gridCell, dataViewBlox);

        if ( properties.length > 0 ) {
            // Create a tooltip with the properties and add to the cell
            StringBuffer buffer = new StringBuffer( 200 );
            for ( int i=0; i < properties.length; i++ ) {
                if ( i > 0 )
                    buffer.append( "\r\n" );

                buffer.append( properties[i].getName() );
                buffer.append( "=" );
                buffer.append( properties[i].getValue() );
            }

            gridCell.setTooltip( buffer.toString() );
        }
    }

    private Property[] getHeaderCellProperties(GridCell gridCell, DataViewBlox dataViewBlox) throws ServerBloxException {
        if ( !(gridCell instanceof GridBrixCellModel ) )
            return new Property[0];

        GridBrixCellModel cell = (GridBrixCellModel)gridCell;

        MDBResultSet results = (MDBResultSet)dataViewBlox.getDataBlox().getResultSet();
        Axis axis = results.getAxis( cell.isColumnHeader() ? Axis.COLUMN_AXIS_ID : Axis.ROW_AXIS_ID );
        Tuple tuple  = axis.getTuple( cell.isColumnHeader() ? cell.getNativeColumn() : cell.getNativeRow() );
        TupleMember tupleMember = tuple.getMember( cell.isColumnHeader() ? cell.getNativeRow() : cell.getNativeColumn() );

        MDBMetaData meta = (MDBMetaData)dataViewBlox.getDataBlox().getMetaData();
        Property[] properties = meta.getPropertiesOfMember( tupleMember.getUniqueName() );
        return properties;
    }
}
