package alphablox;

/**
 * EMailBean.java
 */

import javax.activation.DataHandler;
import javax.activation.FileDataSource;
import javax.activation.URLDataSource;
import javax.mail.*;
import javax.mail.internet.*;

import java.io.File;
import java.lang.reflect.Method;
import java.net.URL;
import java.util.Hashtable;
import java.util.Iterator;
import java.util.Properties;
import java.util.Vector;

/**
 *
 * Copyright 2002 AlphaBlox Corporation. All rights reserved.<BR>
 *
 * This simple EMailBean provides a way to send an email via a JSP Page.
 * Through the use of JSP pages the fields can be set and an email
 * message can be created.
 *
 * @version $Revision: 3 $
 * @author  Dave Messink
 */

public class EMailBean {
    private String from;
    private String replyTo;
    private String to;
    private String cc;
    private String bcc;
    private String subject;
    private String body;

    private Hashtable headers = new Hashtable();
    private Vector bodyParts;
    private String smtpHost;
    private boolean debugOn = false;

    /** Constructor
     */
    public EMailBean() {
        smtpHost = null;
    }

    /** Get the from field
     * @return the from field
     */
    public String getFrom() {
        return from;
    }

    /** Set the from field
     * @param from a string which represents who the email is from
     */
    public void setFrom(String from) {
        this.from = from;
    }

    /** Get the replyTo field
     * @return the replyTo field
     */
    public String getReplyTo() {
        return replyTo;
    }

    /** Set the replyTo field
     * @param replyTo a string which represents who the email is replyTo
     */
    public void setReplyTo(String replyTo) {
        this.replyTo = replyTo;
    }

    /** Get the to field
     * @return the to field
     */
    public String getTo() {
        return to;
    }

    /** Set the to field, comma seperated
     * @param to a string which is comma seperated representing the
     * different email addresses the the email should be sent to
     */
    public void setTo(String to) {
        this.to = to;
    }

    /** Get the cc field
     * @return the cc field
     */
    public String getCc() {
        return cc;
    }

    /** Set the cc field
     * @param cc a string which is comma seperated representing the
     * different email addresses the
     * the email should be cc to
     */
    public void setCc(String cc) {
        this.cc = cc;
    }

    /** Get the cc field
     * @return the cc field
     */
    public String getBcc() {
        return bcc;
    }

    /** Set the bcc field
     * @param bcc a string which is comma seperated representing the
     * different email addresses the
     * the email should be bcc to
     */
    public void setBcc(String bcc) {
        this.bcc = bcc;
    }

    /** Get the subject field
     * @return the subject field
     */
    public String getSubject() {
        return subject;
    }

    /** Set the subject field
     * @param subject a string which represents the subject of the email
     */
    public void setSubject(String subject) {
        this.subject = subject;
    }

    public String getTextBody() {
        return body;
    }

    public void setTextBody(String body) {
        this.body = body;
   }

    protected void addBodyPart(BodyPart bodyPart) {
        if (bodyParts == null)
            bodyParts = new Vector();

        bodyParts.addElement(bodyPart);
    }

    public void attachFile(File file, String filename) throws MessagingException {
        MimeBodyPart bodyPart = new MimeBodyPart();

        FileDataSource fds = new FileDataSource(file.getPath());
        bodyPart.setDataHandler(new DataHandler(fds));
        bodyPart.setFileName(filename);

        addBodyPart(bodyPart);
    }

    public void attachURL(URL url) throws MessagingException {
        MimeBodyPart bodyPart = new MimeBodyPart();

        URLDataSource ds = new URLDataSource(url);
        bodyPart.setDataHandler(new DataHandler(ds));
        addBodyPart(bodyPart);
    }

    /** Get the SMTP host field
     * @return the SMTP Host field
     */
    public String getSMTPHost() {
        return smtpHost;
    }

    /** Set the SMTP host field
     * @param smtpHost a string which represents the SMTP host
     */
    public void setSMTPHost(String smtpHost) {
        this.smtpHost = smtpHost;
    }

    public void setHeader(String header, String value) {
        headers.put(header, value);
    }

    public boolean isDebugOn() {
        return debugOn;
    }

    public void setDebugOn(boolean debugOn) {
        this.debugOn = debugOn;
    }

    /** Sends the email */
    public void send() throws MessagingException, AddressException {
        verifyProperties();

        Properties props = System.getProperties();
        if (smtpHost != null)
            props.put("mail.smtp.host", smtpHost);

        Session session = Session.getDefaultInstance(props, null);
        if (debugOn)
            session.setDebug(true);

        Message message = new MimeMessage(session);

        if (from != null)
            message.setFrom(new InternetAddress(from));
        else
            message.setFrom();

        if (replyTo != null)
            message.setReplyTo(parseAddresses(replyTo));

        message.setRecipients(Message.RecipientType.TO, parseAddresses(to));

        if (cc != null && cc.length() > 0)
            message.setRecipients(Message.RecipientType.CC, parseAddresses(cc));

        if (bcc != null && bcc.length() > 0)
            message.setRecipients(Message.RecipientType.BCC, parseAddresses(bcc));

        message.setSubject(subject);

        MimeMultipart content = getContent();
        message.setHeader("Content-Type", content.getContentType());
        message.setContent(content);

        Iterator headerList = headers.keySet().iterator();
        while (headerList.hasNext()) {
            String header = (String) headerList.next();
            message.setHeader(header, (String) headers.get(header));
        }

        message.setHeader("MIME-Version", "1.0");
        message.setSentDate(new java.util.Date());

        if (debugOn) {
            try {
                System.out.println("SENDING EMAIL MESSAGE:");
                message.writeTo(System.out);
            } catch (Exception exc) {
                System.out.println("couldn't write message!");
            }
        }

        Transport.send(message);
    }

    private void verifyProperties() throws RuntimeException {
        if (to == null || to.length() == 0)
            throw new IllegalStateException("The 'to' property is not set correctly");

        if (smtpHost == null || smtpHost.length() == 0)
            throw new IllegalStateException("The 'SMTPHost' property is not set correctly");
    }

    private MimeMultipart getContent() throws MessagingException {
        MimeMultipart multipart = new MimeMultipart("related");

        if (body != null) {
            BodyPart bodyPart = new MimeBodyPart();
            bodyPart.setText(body);
            multipart.addBodyPart(bodyPart);
        }

        if (bodyParts != null) {
            Iterator partsList = bodyParts.iterator();

            while (partsList.hasNext())
                multipart.addBodyPart((BodyPart) partsList.next());
        }

        return multipart;
    }

    /* handles both ',' and ';' delimited addresses */
    private InternetAddress[] parseAddresses(String addresses) throws AddressException {
        return InternetAddress.parse(addresses.replace(';', ','), false);
    }
}
