package alphablox;

import com.alphablox.net.HttpRequest;
import com.alphablox.net.HttpResponse;
import com.alphablox.net.URLFactory;

import javax.servlet.http.HttpServletRequest;
import java.io.*;
import java.util.Enumeration;
import java.util.HashSet;
import java.util.Set;

/**
 * HTMLFile.java
 */

/**
 *
 * Copyright 2002 AlphaBlox Corporation. All rights reserved.<BR>
 *
 * @version $Revision: 1 $
 * @author  Dave Messink
 */

public class HTMLFile {
    private HttpServletRequest request;
    private String urlPath;
    private String contentType;

    HttpResponse httpResponse;

    private static Set ignoredHeaders = new HashSet();

    static {
        // Add headers from the webserver/WAS that should NOT be sent
        // as part of the http request
        ignoredHeaders.add("content-length");
        ignoredHeaders.add("keep-alive");
        ignoredHeaders.add("transfer-encoding");
        ignoredHeaders.add("connection");
    }

    public HTMLFile(HttpServletRequest request, String urlPath)
            throws IOException {
        this.request = request;
        this.urlPath = urlPath;
        contentType = null;
        httpResponse = null;
    }

    public InputStream getInputStream() throws IOException {
        URLFactory urlFactory = getURLFactory();
        HttpRequest httpRequest = new HttpRequest(urlFactory);

        setRequestHeaders(httpRequest);

        if (urlFactory.hasParameters())
            httpResponse = httpRequest.post();
        else
            httpResponse = httpRequest.get();

        if (httpResponse.getResponseCode() != HttpResponse.HTTP_OK) {
            throw new RuntimeException("Connection to " + urlFactory + " failed.");
        }

        contentType = httpResponse.getContentType();

        return httpResponse.getInputStream();
    }

    public char[] getCharArray() throws IOException {
        InputStream inputStream = getInputStream();
        InputStreamReader isReader = new InputStreamReader(inputStream, "UTF-8");
        BufferedReader reader = new BufferedReader(isReader);
        CharArrayWriter writer = new CharArrayWriter(4 * 1024);

        String line;
        while ((line = reader.readLine()) != null)
        {
	    writer.write("\r\n");
            writer.write(line);
        }

        httpResponse.disconnect();
        return writer.toCharArray();
    }

    public String getContentType() {
        return contentType;
    }

    private URLFactory getURLFactory() {
        URLFactory urlFactory = new URLFactory(request);

        urlFactory.setProtocol(request.getScheme());
        urlFactory.setHost(request.getServerName());
        urlFactory.setPort(request.getServerPort());

        if (urlPath.startsWith("/"))
            urlFactory.setURL(urlPath);
        else
            urlFactory.setPath(urlPath);

        return urlFactory;
    }

    private void setRequestHeaders(HttpRequest httpRequest) {
        Enumeration headerList = request.getHeaderNames();

        while (headerList.hasMoreElements()) {
            String header = (String) headerList.nextElement();

            if (ignoredHeaders.contains(header.toLowerCase()))
                continue;

            Enumeration valueList = request.getHeaders(header);

            while (valueList.hasMoreElements()) {
                String value = (String) valueList.nextElement();
                httpRequest.addHeader(header, value);
            }
        }
    }
}
