/* */

import org.apache.avalon.framework.logger.ConsoleLogger;
import org.apache.avalon.framework.logger.Logger;
import org.apache.fop.apps.Driver;
import org.apache.fop.messaging.MessageHandler;
import org.xml.sax.InputSource;
import org.xml.sax.XMLReader;

import javax.servlet.ServletException;
import javax.servlet.http.HttpServlet;
import javax.servlet.http.HttpServletRequest;
import javax.servlet.http.HttpServletResponse;
import javax.xml.parsers.SAXParserFactory;
import java.io.ByteArrayInputStream;
import java.io.ByteArrayOutputStream;

/**
 * Example
 *
 * <code>post</code> an fo tree (attribute name = 'fo') to http://servername/PdfServer/FopServlet
 *
 * Running: you will need in the WEB-INF/lib/ directory:
 * - fop.jar
 * - xalan.jar
 * - avalon.jar
 */
public class FopServlet extends HttpServlet {
    public static final String FO_REQUEST_PARAM = "fo";
    Logger log = null;

    public void doGet( HttpServletRequest request, HttpServletResponse response ) throws ServletException {
        doPost( request, response );
    }

    public void doPost( HttpServletRequest request, HttpServletResponse response ) throws ServletException {
        if( log == null ) {
//            log = new NullLogger();
            log = new ConsoleLogger( ConsoleLogger.LEVEL_WARN );
        }

        try {
            String fo = request.getParameter( FO_REQUEST_PARAM );
            if( fo != null ) {
                try {
                    renderFO( new InputSource( new ByteArrayInputStream( fo.getBytes() ) ), response );
                }
                catch( ServletException e ) {
                    String errorMessage = e.getMessage();
                    response.getWriter().println( getErrorMessage( "Couldn't convert the XSL-FO tree to PDF.",
                            ( errorMessage != null ) ? errorMessage : "" ) );
                }
            }
            else {
                response.getWriter().println( getErrorMessage( "No 'fo' request parameter given.", "" ) );
            }
        }
        catch( Exception ex ) {
            throw new ServletException( ex );
        }
    }

    private String getErrorMessage( String shortMessage, String longMessage ) {
        StringBuffer buffer = new StringBuffer();

        buffer.append( "<html>\n" );
        buffer.append( "<head>" );
        buffer.append( "<title>PDF</title>" );
        buffer.append( "</head>\n" );
        buffer.append( "<body>\n" );
        buffer.append( "<h1>FopServlet Error</h1>" );
        buffer.append( "<h3>" ).append( shortMessage ).append( "</h3>\n" );
        buffer.append( "<h5>" ).append( longMessage ).append( "</h5>\n" );
        buffer.append( "</body>\n" );
        buffer.append( "</html>" );

        return buffer.toString();
    }

    /**
     * Renders an FO inputsource into a PDF file which is written
     * directly to the response object's OutputStream
     */
    public void renderFO( InputSource foFile, HttpServletResponse response ) throws ServletException {
        try {
            ByteArrayOutputStream out = new ByteArrayOutputStream();
            // tell the browser this is going to be PDF
            response.setContentType( "application/pdf" );
            // Tell FOP to use our log handler
            MessageHandler.setScreenLogger( log );
            Driver driver = new Driver();
            driver.setLogger( log );
            driver.setRenderer( Driver.RENDER_PDF );
            driver.setOutputStream( out );
            driver.render( createSaxParser(), foFile );

            byte[] content = out.toByteArray();
            response.setContentLength( content.length );
            response.getOutputStream().write( content );
            response.getOutputStream().flush();
        }
        catch( Exception ex ) {
            throw new ServletException( ex );
        }
    }

    public XMLReader createSaxParser() throws Exception {
        SAXParserFactory factory = SAXParserFactory.newInstance();
        factory.setNamespaceAware( true );
        return factory.newSAXParser().getXMLReader();
    }
}
