package com.alphablox.fts;

/*
 * Copyright (c) 2002-2003 Alphablox Corporation. All rights reserved.
 */


import java.util.HashMap;
import java.util.Vector;

import com.alphablox.blox.DataBlox;
import com.alphablox.blox.ServerBloxException;
import com.alphablox.blox.data.mdb.*;
import dhtmlQueryBuilder.QueryBuilderBean;

public class QueryGenerator
{
    private boolean isEssbase = false;
    private boolean isMDX = false;
    private boolean isSupported = false;
    private DataBlox dataBlox;
    private String dataBaseName = "";

    public QueryGenerator( )
    {
    }

    public QueryGenerator( DataBlox db ) throws Exception
    {
        init( db );
    }

    public void init( DataBlox db ) throws Exception
    {
        final String dataBaseName = getDataBaseName();

        this.setDataBlox(db);
        this.setDataBaseName(db.getMetaData().getDatabaseProductName());
        this.setEssbase(QueryBuilderBean.DS_DB2FOROLAP.equals( dataBaseName ) || QueryBuilderBean.DS_HYPERION.equals( dataBaseName ));
        this.setMDX(QueryBuilderBean.DS_MSAS.equals( dataBaseName ) || QueryBuilderBean.DS_SAP_BW.equals( dataBaseName ) || QueryBuilderBean.DS_SSAS.equals( dataBaseName ));
    }

    public String getDataBaseName() {
        return dataBaseName;
    }

    public void setDataBaseName(String dataBaseName) {
        this.dataBaseName = dataBaseName;
    }

    public DataBlox getDataBlox() {
        return dataBlox;
    }

    public void setDataBlox(DataBlox dataBlox) {
        this.dataBlox = dataBlox;
    }

    public boolean isEssbase() {
        return isEssbase;
    }

    public void setEssbase(boolean essbase) {
        isEssbase = essbase;
        if(isEssbase)
            this.setSupported(true);
    }

    public boolean isMDX() {
        return isMDX;
    }

    public void setMDX(boolean isMDX) {
        this.isMDX = isMDX;

        if(this.isMDX)
            this.setSupported(true);
    }

    public boolean isSupported() {
        return isSupported;
    }

    public void setSupported(boolean supported) {
        isSupported = supported;
    }

    /**
     *  *	Generate a query string based on the current result set.
     *
     *  *	@return String query
     *  *	@exception com.alphablox.blox.ServerBloxException
     */

    public String generateQuery( ) throws ServerBloxException
    {
        if ( !isEssbase )
        {
            return dataBlox.generateQuery();
        }

        // Generate an ESSBASE query

        StringBuffer query = new StringBuffer( 3000 );

        MDBResultSet results = (MDBResultSet)dataBlox.getResultSet();

        if ( dataBlox.isUseAliases() )
            query.append( "{OUTALTNAMES} " );

        String pages = getPages( results );		// get the page dimension members if any..
        if ( pages != "null" && pages.length() > 0 ) {
            query.append( pages );
            query.append( " " );
        }

        String rows = getRows( results );		// get the row dimension members if any..
        if ( rows != "null" && rows.length() > 0 ) {
            query.append( rows );
            query.append( " " );
        }

        String columns = getColumns( results );	// get the column dimension members if any..
        if ( columns != null && columns.length() > 0 ) {
            query.append( columns );
            query.append( " " );
        }

        query.append( "!" );

        return query.toString();
    }

    /**
     * Generate the row report spec (for ESSBASE ONLY) for the current query.
     *
     * @return String row spec or null
     * @throws ServerBloxException
     */
    public String generateRowSpec( )  throws ServerBloxException
    {
        String rowSpec = "";

        if( isSupported && !dataBlox.getQuery().equals("") )
        {
            rowSpec+=getRows( (MDBResultSet)dataBlox.getResultSet() );

            if ( !rowSpec.equals("") )
            {
                if( this.isMDX() )
                {
                    if(dataBlox.isSuppressDuplicates())
                    {
                        rowSpec = " Distinct(" + rowSpec + " )";
                    }
                    rowSpec += " ON ROWS";
                }
            }
        }
        return rowSpec;
    }

    /**
     * Generate the column report spec (for ESSBASE ONLY) for the current query.
     *
     * @return String column spec or null
     * @throws ServerBloxException
     */
    public String generateColumnSpec( )  throws ServerBloxException
    {
        String colSpec = "";
        if ( isSupported && !dataBlox.getQuery().equals("") )
        {
            colSpec+=getColumns( (MDBResultSet)dataBlox.getResultSet() );

            if ( !colSpec.equals("") )
            {
                if( this.isMDX() )
                {
                    if(dataBlox.isSuppressDuplicates())
                    {
                        colSpec = " Distinct(" + colSpec + " )";
                    }
                    colSpec += " ON COLUMNS";
                }
            }
        }
        return colSpec;
    }

    /**
     * Generate the page report spec (for ESSBASE ONLY) for the current query.
     *
     * @return String page spec or null
     * @throws ServerBloxException
     */
    public String generatePageSpec( )  throws ServerBloxException
    {
        String pageSpec = "";
        if( isSupported && !dataBlox.getQuery().equals("") )
            pageSpec=getPages( (MDBResultSet)dataBlox.getResultSet() );
        return pageSpec;
    }

    // get the row elements from the grid
    private String getRows( MDBResultSet results )
    {
        Axis axis = results.getAxis( Axis.ROW_AXIS );
        if ( axis == null )
        {
            return "";
        }
        AxisDimension[] dimensions = axis.getDimensions();
        if ( dimensions.length == 0 )
            return "";

        StringBuffer spec = new StringBuffer( 2000 );
        if(this.isEssbase()) {
            spec.append( "<ROW (" );

            for ( int i=0; i < dimensions.length; i++ )
            {
                if ( i > 0 )
                    spec.append( ", " );

                spec.append( "'" );
                spec.append( dimensions[i].getDisplayName() );
                spec.append( "'" );
            }
            spec.append( ")" );
        }

        String mspec = createMemberSpec( axis, dimensions );

        if ( mspec != null )                      					// Abbreviated form
            spec.append( mspec );
        else 														// Cononical form
            addAllResultMembers( spec, axis, dimensions );

        return spec.toString();
    }

    // build up the pages portion of the query string
    private String getPages( MDBResultSet results )
    {
        Axis axis = results.getAxis( results.getSlicerAxisIndex() );

        if ( axis == null )
        {
            return " ";
        }

        AxisDimension[] dimensions = axis.getDimensions();

        if ( dimensions.length == 0 )
            return " ";

        StringBuffer spec = new StringBuffer( 500 );
        Tuple t = axis.getTuple( 0 );

        for ( int i=0; i < t.getMemberCount(); i++ )
        {
            TupleMember member = t.getMember( i );

            spec.append( "'" );
            spec.append( member.getDisplayName( ) );
            spec.append( "' " );
        }

        return spec.toString();
    }

    // build the columns part of the query string
    private String getColumns( MDBResultSet results )
    {
        Axis axis = results.getAxis( Axis.COLUMN_AXIS );

        if ( axis == null )
        {
            return null;
        }

        AxisDimension[] dimensions = axis.getDimensions();

        if ( dimensions.length == 0 )
            return null;

        StringBuffer spec = new StringBuffer( 2000 );
        if(this.isEssbase())
            spec.append( "<COLUMN (" );

        // loop through every member in every dimension by iterating through all of
        // the tuples and putting them into an appropriate associative array. I.e.
        // all the members of dimension 0 go into an associative array stored in
        // dimMemberArrays[0]

        String[] dimNames        = new String[dimensions.length];	// hold dimension names
        HashMap[] dimMemberArrays = new HashMap[dimensions.length];	// holds unique members from each dimension
        boolean isSymmetrical = true;
        int symColumnCount  = 1;				// the number of cols if the result set is symmetric
        int tupleCount = 0;

        // iterate through all of the dimensions on the axis
        if(this.isEssbase()) {
            for ( int i=0; i < dimensions.length; i++ )	{
                if ( i > 0 )
                    spec.append( ", " );

                spec.append( "'" );
                spec.append( dimensions[i].getDisplayName() );
                spec.append( "'" );

                // iterate through all of the columns on the grid in order to build up
                // associative arrays of the unique member names on each dimension

                dimNames[i] = dimensions[i].getDisplayName();
                dimMemberArrays[i] = new HashMap();

                for( int j=0; j < axis.getTupleCount(); j++ )
                {
                    String member = axis.getTuple( j ).getMember( i ).getDisplayName();
                    dimMemberArrays[i].put( member, member );
                }
            }

            spec.append( ") " );

            //  iterate through the array of members for each dimension. if the report is
            //  symmetric, the product of the member counts for each dimension will equal the
            //  number of columns
            tupleCount = axis.getTupleCount();

            for( int i = 0; i < dimensions.length; i++ )
                symColumnCount = symColumnCount * dimMemberArrays[i].size();

            // check to see is the report is asymmetric
            isSymmetrical = ( symColumnCount == tupleCount );
            spec.append( (isSymmetrical) ? "<SYM" : "<ASYM" );
        }
        // generate the member lists for each dimension on the axis

        if( isSymmetrical )
        {
            String mspec = createMemberSpec( axis, dimensions );

            if ( mspec != null )                      					// Abbreviated form
                spec.append( mspec );
            else 														// Cononical form
                addAllResultMembers( spec, axis, dimensions );
        }
        else {
            //  build up an asymmetric report string. in order to do this, you need
            //  to list tuples instead of individual members for every tuple on the
            //  column axis so, iterate through all of the column tuples and add
            //  them into the query string

            for( int j=0; j < dimensions.length; j++ ) {
                for( int i = 0; i < tupleCount; i++ ) {
                    Tuple tuple = axis.getTuple( i );
                    TupleMember member = tuple.getMember( j );

                    if(member.getDisplayName().indexOf("Q-T-D")>-1) {
                        spec.append( " " );
                        spec.append( member.getDisplayName() );
                    }
                    else {
                        spec.append( " '" );
                        spec.append( member.getDisplayName() );
                        spec.append( "'" );
                    }
                }
            }
        }

        return spec.toString();
    }

    private String createMemberSpec( Axis axis, AxisDimension[] dimensions )
    {
        try
        {
            MDBMetaData meta = (MDBMetaData)dataBlox.getMetaData();

            StringBuffer memberspec = new StringBuffer( 1000 );

            if(this.isMDX()) {
                for( int dim = 1; dim < dimensions.length; dim++ ) {
                    memberspec.append(" crossjoin( ");
                }
            }

            for( int dim = 0; dim < dimensions.length; dim++ )
            {
                TupleMember[] members = extractUniqueMemberList( axis, dim );
                if(this.isMDX())
                    memberspec.append("{");
                for ( int j=0; j < members.length; j++ ) {
                    //System.out.println("Member:"+j);
                    Member member = meta.resolveMember( members[j].getUniqueName() );

                    // JGD This check used to throw an exception on the assumption that
                    // if a member didn't resolve, there must be something wrong.  Now
                    // the querybuilder can have calculations, so that's no longer true.
                    if ( member == null )
                    {
                        continue;
                    }

                    Member parent = member.getParent();

                    // If a top level member in the result set (i.e. no parents), then build the
                    // reportspec hierarchy
                    if ( parent == null || findMember( members, parent ) == -1 )
                        createMemberHierarchySpec( memberspec, members, member, meta );
                }
                if(this.isMDX())
                    if(dimensions.length==1)
                        memberspec.append("} ");
                    else if(dim==0)
                        memberspec.append("}, ");
                    else
                        memberspec.append("}) ");
            }

            return memberspec.toString();
        }
        catch ( Exception e )
        {
            e.printStackTrace();
            return null;
        }
    }

    private int findMember( TupleMember[] members, Member target )
    {
        if ( target == null )
            return -1;

        for ( int i=0; i < members.length; i++ )
            if ( members[i].getUniqueName().equals( target.getUniqueName() ) )
                return i;

        return -1;
    }

    private void createMemberHierarchySpec( StringBuffer memberspec, TupleMember[] members, Member parent, MDBMetaData meta ) throws Exception
    {
        Member[] kids = extractChildren(parent, members );
        String msasChildren = "";
        if ( kids.length > 0 ) 											// No kids ... just the parent
        {
            if ( kids.length == parent.getChildren().length &&
                 !hasGrandChildren( members, kids) ) { 			// Parent and all children
                if(this.isEssbase())
                    memberspec.append( " <ICHILD" );
                if(this.isMDX())
                    msasChildren = ".Children";
            }
            else   														// Parent and some children
                for ( int k=0; k <  kids.length; k++ ) {
                    createMemberHierarchySpec( memberspec, members, kids[k], meta );
                    if(this.isMDX())
                        memberspec.append( "," );
                }
        }

        if (this.isEssbase()) {
            memberspec.append( " '" );
            memberspec.append( parent.getDisplayName() );
            memberspec.append( "'" );
        }
        if(this.isMDX()) {
            memberspec.append( parent.getUniqueName() );
            if(!msasChildren.equals("")) {
                memberspec.append( ", "+parent.getUniqueName() );
                memberspec.append( msasChildren );
            }
            //System.out.println(memberspec.toString());
        }
    }

    private boolean hasGrandChildren(  TupleMember[] members, Member[] kids) throws Exception
    {
        for ( int k=0; k <  kids.length; k++ ) {
            Member[] grandkids = extractChildren(kids[k], members );

            if ( grandkids .length > 0 )
                return true;
        }

        return false;
    }

    private Member[] extractChildren(Member parent, TupleMember[] members ) throws Exception
    {
        Vector kids = new Vector( );
        Member[] allChildren = (Member[])parent.getChildren().clone();

        for ( int i=0; i < members.length && kids.size() < allChildren.length; i++ )
            for ( int j=0; j < allChildren.length; j++ )
                if ( allChildren[j] != null &&
                     allChildren[j].getUniqueName().equals( members[i].getUniqueName() ) ) {
                    kids.add( allChildren[j] );
                    allChildren[j] = null;
                    break;
                }

        Member[] result = new Member[ kids.size() ];
        kids.copyInto( result );
        return result;
    }

    private TupleMember[] extractUniqueMemberList( Axis axis, int dimensionIndex )
    {
        Vector m = new Vector( );
        HashMap duplicates = new HashMap();
        int count = axis.getTupleCount();

        for ( int i=0; i < count; i++ ) {
            TupleMember member = axis.getTuple( i ).getMember( dimensionIndex );
            String name = member.getUniqueName();

            if ( duplicates.get( name ) == null ) {
                m.add( member );
                duplicates.put( name, name );
            }
        }

        TupleMember[] result = new TupleMember[ m.size() ];
        m.copyInto( result );
        return result;
    }

    private void addAllResultMembers( StringBuffer spec, Axis axis, AxisDimension[] dimensions )
    {
        int tupleCount = axis.getTupleCount();

        HashMap duplicates = new HashMap( );

        for( int i = 0; i < dimensions.length; i++ )
        {
            duplicates.clear( );

            for( int j = 0; j < tupleCount; j++ )
            {
                TupleMember member = axis.getTuple( j ).getMember( i );

                if ( duplicates.get( member ) == null ) {
                    duplicates.put( member, member );

                    spec.append( " '" );
                    spec.append( member.getDisplayName() );
                    spec.append( "'" );
                }
            }
        }
    }
}
