package com.alphablox.fts;

/*
 * Copyright (c) 2002-2003 Alphablox Corporation. All rights reserved.
 */


import com.alphablox.blox.DataBlox;
import com.alphablox.blox.ServerBloxException;
import com.alphablox.blox.data.mdb.*;

import java.util.*;

import dhtmlQueryBuilder.AppContext;

public class asymColTableBean extends QueryGenerator {

    private ArrayList dimArray;
    private AppContext mAppContext;


    public asymColTableBean( AppContext appContext ) {
        this.mAppContext = appContext;
    }

    public asymColTableBean( DataBlox db, AppContext appContext ) throws Exception	{
        super( db );
        this.mAppContext = appContext;
    }

    public ArrayList getDimArray() {
          return dimArray;
    }


    /**
     *  *	Generate a query string based on the current result set.
     *
     *  *	@return String query
     *  *	@exception com.alphablox.blox.ServerBloxException
     */


    /**
     * Generate the column report spec (for ESSBASE ONLY) for the current query.
     *
     * @return String column spec or null
     * @execption ServerBloxException
     */
    public String generateColumnTable( )  throws ServerBloxException
    {
        String returnValue = "";
        if ( this.isEssbase() )
        {
            if ( !this.getDataBlox().getQuery().equals("") )
            {
                returnValue = getColumnTable( (MDBResultSet)this.getDataBlox().getResultSet() );
            }
            else
            {
                returnValue = mAppContext.getString("ASSYM_NO_COLUMNS");
            }
        }
        else
        {
            returnValue = mAppContext.getString("ASSYM_DATABASE_NOT_SUPPORTED", this.getDataBaseName() );
        }
        return returnValue;
    }


    // If a tuple contains a calculation, then you can't remove it from the query.
    private boolean canRemoveTuple( Axis axis, int index) throws ServerBloxException
    {
        MDBMetaData meta = (MDBMetaData)this.getDataBlox().getMetaData();
        AxisDimension[] dimensions = axis.getDimensions();
        if ( dimensions.length == 0 )
        {
            return false;
        }

        for ( int i=0; i < dimensions.length; i++ )
        {
            Tuple tuple = axis.getTuple( index - 1 );
            TupleMember member = tuple.getMember( i );
            Member cubeMember = meta.resolveMember( member.getUniqueName() );
            if ( cubeMember == null )
            {
                // Calculation. Can't hide a tuple with a calc.
                return false;
            }
        }
        return true;
    }

    // build the columns part of the query string
    private String getColumnTable( MDBResultSet results ) throws ServerBloxException
    {
        Axis axis = results.getAxis( Axis.COLUMN_AXIS );

        if ( axis == null )
        {
            return  mAppContext.getString("ASSYM_NO_COLUMNS");
        }

        AxisDimension[] dimensions = axis.getDimensions();

        if ( dimensions.length == 0 )
        {
            return  mAppContext.getString("ASSYM_NO_COLUMNS");
        }
        StringBuffer spec = new StringBuffer( 6000 );
        spec.append( "<table cellspacing=\"0\" cellpadding=\"0\" border=\"0\" class=\"tableColAsym\">" );

        // loop through every member in every dimension by iterating through all of
        // the tuples and putting them into an appropriate associative array. I.e.
        // all the members of dimension 0 go into an associative array stored in
        // dimMemberArrays[0]
        // What's happening here:
        // The member dimArray keeps track of the removeable tuples that are displayed in
        // the assymetric query builder.  It does so by storing an ordered list of members
        // for each dimension.
        // In addition to building up the dimArray, we are also building the html content
        // that will be displayed.  This consists of a row of checkboxes and a cell for
        // each tuplemember being displayed(basically one column per tuple).  The value of each
        // checkbox is the index of the tuple with which it is associated.
        int tupleCount = axis.getTupleCount();
        boolean [] removeableTuples = new boolean [tupleCount];
        boolean foundATupleToRemove = false;

        spec.append( "<tr><td class=\"tdHideShowLabel\">Visible</td>" );
        int tuplePosition = 1;
        // go through the tuples and build the list of indexes. Exclude tuples with calcs in them.
        for( int i=1; i<=tupleCount; i++)
        {
            removeableTuples[i-1] = canRemoveTuple(axis, i);
            if ( removeableTuples[i-1] )
            {
                foundATupleToRemove = true;
                spec.append( "<td class=\"tdHideShow\"><input name=\""+tuplePosition+"\" type=\"checkbox\" value=\"true\" checked></td>" );
                tuplePosition++;
            }
        }

        if ( !foundATupleToRemove )
        {
            return  mAppContext.getString("ASSYM_NO_COLUMNS");
        }

        spec.append( "</tr>" );
        this.dimArray = new ArrayList(dimensions.length);
        ArrayList tupleArray;
        // iterate through all of the dimensions on the axis
        for ( int i=0; i < dimensions.length; i++ )
        {
            spec.append( "<tr>" );
            tupleArray = new ArrayList();
            String memberName = "'"+dimensions[i].getDisplayName()+"'";

            if ( i > 0 )
            {
                memberName=", "+memberName;
            }
            spec.append( "<td nowrap class=\"tdDimName\">"+dimensions[i].getDisplayName()+"</td>" );
            tupleArray.add( memberName );

            for( int j = 0; j < tupleCount; j++ )
            {
                if ( removeableTuples[j] )
                {
                    Tuple tuple = axis.getTuple( j );
                    TupleMember member = tuple.getMember( i );

                    memberName = "";
                    if(member.getDisplayName().indexOf("Q-T-D")>-1)
                    {
                        memberName = " "+member.getDisplayName();
                    }
                    else
                    {
                        memberName = " '" + member.getDisplayName() + "'";
                    }
                    spec.append( "<td nowrap class=\"tdTuple\">"+member.getDisplayName()+"</td>" );
                    tupleArray.add( memberName );
                }
            }
            this.dimArray.add(tupleArray);
            spec.append( "</tr>" );
        }

        //  iterate through the array of members for each dimension. if the report is
        //  symmetric, the product of the member counts for each dimension will equal the
        //  number of columns
        spec.append( "</table>" );

        return spec.toString();
    }


}
