/*
 * IBM Confidential
 * OCO Source Materials
 * 5724-L14
 * (c) Copyright IBM Corp. 2005
 * The source code for this program is not published or otherwise divested
 *   of its trade secrets, irrespective of what has been deposited with the
 *   U.S. Copyright Office.
 *
 * User: jdouglas
 * Date: Mar 8, 2004
 * Time: 2:40:52 PM
 */

package dhtmlQueryBuilder;

import com.alphablox.blox.BloxContext;
import com.alphablox.blox.BloxContextFactory;

import javax.servlet.ServletContext;
import javax.servlet.http.HttpServletRequest;
import javax.servlet.http.HttpServletResponse;
import javax.servlet.jsp.PageContext;
import java.text.MessageFormat;
import java.util.Locale;
import java.util.MissingResourceException;
import java.util.ResourceBundle;


public class AppContext {
    private static final String QB_BUNDLE_NAME = "dhtmlQueryBuilder.strings";

    private ServletContext servletContext;
    private ResourceBundle stringTable;

    public void init(PageContext pageContext) throws Exception
    {
        servletContext = pageContext.getServletContext();

        try {
            HttpServletRequest httpRequest = (HttpServletRequest) pageContext.getRequest();
            HttpServletResponse httpResponse = (HttpServletResponse) pageContext.getResponse();

            BloxContext bloxContext = BloxContextFactory.getBloxContext(httpRequest, httpResponse);
            stringTable = choosePreferredBundle(bloxContext.getLocales());
        }
        catch(MissingResourceException mre) {
              System.out.println("Missing Resource: " + mre.getMessage());
        }
    }

    public String getResourcePath(String subPath)
    {
        String mypath = servletContext.getRealPath(subPath);
        return mypath;
    }

    public ResourceBundle getSQLTable()
    {
        return stringTable;
    }

    public String getString(String key)
    {
        return stringTable.getString(key);
    }

    public String getString(String key, String arg1)
    {
        return getString(key, new Object[]{arg1});
    }

    public String getString(String key, Object[] argList)
    {
        String resourceString = stringTable.getString(key);
        String message = MessageFormat.format(resourceString, argList);
        return message;
    }


    protected Locale getPreferredLocale() {
        return stringTable.getLocale();
    }

    private ResourceBundle choosePreferredBundle(Locale[] locales) {
        ResourceBundle preferred = null;
        ResourceBundle worstCase = null;

        // Algorithm to use the adapter to find a matching locale
        if (locales != null) {
            for (int i = 0; i < locales.length; i++) {
                if (locales[i] == null)
                    continue;

                ResourceBundle choice = ResourceBundle.getBundle(QB_BUNDLE_NAME, locales[i]);
                if (choice == null)
                    continue;

                Locale locale = choice.getLocale();

                // If an exact match we are done
                if (locale.equals(locales[i]))
                    return choice;

                // Check for best case
                if (locale.getLanguage().equals(locales[i].getLanguage())) {
                    if ("en".equals(locale.getLanguage()))      // Match any Enlish variant to the based resources
                        return choice;

                    // If country/lang matched and this is the first then use it
                    if (preferred == null && locale.getCountry().equals(locales[i].getCountry()))
                        preferred = choice;

                    else if (worstCase == null)
                        worstCase = choice;
                }
            }
        }

        if (preferred == null)
            preferred = worstCase;

        // If we still have no resource at this point either no locale was given or no language matches
        if (preferred == null)
            preferred = ResourceBundle.getBundle(QB_BUNDLE_NAME, Locale.ENGLISH);

        return preferred;
    }
}
