package dhtmlQueryBuilder;

import com.alphablox.blox.uimodel.GridBrixModel;
import com.alphablox.blox.uimodel.core.Component;
import com.alphablox.blox.uimodel.core.Image;
import com.alphablox.blox.uimodel.core.ModelException;
import com.alphablox.blox.uimodel.core.event.ClickEvent;
import com.alphablox.blox.uimodel.core.event.ComponentRebuiltNotify;
import com.alphablox.blox.uimodel.core.event.IEventHandler;
import com.alphablox.blox.uimodel.core.grid.GridCell;
import com.alphablox.blox.uimodel.core.grid.GridColumn;
import com.alphablox.blox.uimodel.core.grid.GridRow;

import java.util.Iterator;

/**
 * Title:
 * Description:
 * Copyright:    Copyright (c) 2002-2003 Alphablox Corporation.
 * Company:
 * @author
 * @version 1.0
 */

public class TableSelectEventHandler implements IEventHandler
{
    private AppContext mAppContext;
    private QueryBuilderBean mQueryBuilder;

    public TableSelectEventHandler( QueryBuilderBean qb, AppContext appContext )
    {
        this.mAppContext = appContext;
        this.mQueryBuilder = qb;
    }

    public boolean handleComponentRebuiltNotify( ComponentRebuiltNotify event ) throws Exception
    {
        if ( mQueryBuilder.isRelational() )
        {
            Component component = event.getComponent( );

            if ( component instanceof GridBrixModel )
                addTableLinks( (GridBrixModel)component );
        }
        return false;
    }

    public boolean handleClickEvent( ClickEvent event ) throws Exception
    {
        if ( mQueryBuilder.isRelational() )
        {
            Component component = event.getComponent( );
            if ( component instanceof Image )
            {
                String fileName = ((Image) component).getImageURL();
                // Bug 18935
                if ( fileName.equals("images/square.gif"))
                {
                    String table = ((GridCell)component.getParent()).getValue();
                    mQueryBuilder.setQueryText( mAppContext.getString("SQL_DEFAULT_QUERY", table),false);
                    mQueryBuilder.executeQuery();
                }
            }
        }
        return false;
    }

    private void addTableLinks( GridBrixModel grid ) throws ModelException
    {
        try
        {
            Iterator iterator = grid.columnIterator();
            while ( iterator.hasNext() )
            {
                GridColumn column = (GridColumn)iterator.next();
                // I'm assuming the column name is unique across the different data sources.
                // For example, the Oracle column is 'TNAME'.  If the DB2 catolog query returned
                // a column named 'TNAME', then I'd be in trouble.  So far, we're ok.
                if ( QueryBuilderBean.SQLSERVER_CATALOG_TABLE_NAME.equalsIgnoreCase(column.getCell(0).getValue()) ||
                     QueryBuilderBean.DB2_CATALOG_TABLE_NAME.equalsIgnoreCase(column.getCell(0).getValue()) ||
                     QueryBuilderBean.ORACLE_CATALOG_TABLE_NAME.equalsIgnoreCase(column.getCell(0).getValue())
                   )
                {
                    for ( int i=0; i < grid.getRowCount(); i++ )
                    {
                        GridRow row = grid.getRow( i );
                        if ( !row.isHeader() )
                        {
                            GridCell cell = row.getCell(column.getIndex());
                            Image image = new Image();
                            image.setImageURL("images/square.gif");
                            image.setTooltip("Select Table");
                            image.setClickable(true);
                            cell.add( image );
                        }
                    }
                }
            }
        }
        catch (ModelException e)
        {
            System.out.println( mAppContext.getString("EXCEPTION_EVENT_HANDLER_FAILED", e.getMessage()) );
        }
    }
}
