package dhtmlQueryBuilder;

import javax.servlet.http.HttpServletRequest;
import javax.servlet.http.HttpServletResponse;
import java.awt.*;
import java.lang.reflect.Field;
import java.lang.reflect.InvocationTargetException;
import java.lang.reflect.Method;
import java.util.HashSet;

import com.alphablox.blox.BloxContext;
import com.alphablox.blox.BloxContextFactory;
import com.alphablox.blox.ChartBloxUnavailableException;
import com.alphablox.blox.DataLayoutBloxUnavailableException;
import com.alphablox.blox.GridBlox;
import com.alphablox.blox.GridBloxUnavailableException;
import com.alphablox.blox.InvalidParameterException;
import com.alphablox.blox.PageBloxUnavailableException;
import com.alphablox.blox.PresentBlox;
import com.alphablox.blox.ServerBloxException;
import com.alphablox.blox.ServerBloxMissingResourceException;
import com.alphablox.blox.ToolbarBloxUnavailableException;
import com.alphablox.blox.data.mdb.Dimension;
import com.alphablox.blox.uimodel.ModelConstants;
import com.alphablox.blox.uimodel.PresentBloxModel;
import com.alphablox.blox.uimodel.core.Button;
import com.alphablox.blox.uimodel.core.Component;
import com.alphablox.blox.uimodel.core.ComponentContainer;
import com.alphablox.blox.uimodel.core.Dialog;
import com.alphablox.blox.uimodel.core.DialogController;
import com.alphablox.blox.uimodel.core.Edit;
import com.alphablox.blox.uimodel.core.HorizontalLayout;
import com.alphablox.blox.uimodel.core.Menu;
import com.alphablox.blox.uimodel.core.MenuItem;
import com.alphablox.blox.uimodel.core.Menubar;
import com.alphablox.blox.uimodel.core.ModelException;
import com.alphablox.blox.uimodel.core.Spacer;
import com.alphablox.blox.uimodel.core.VerticalLayout;
import com.alphablox.blox.uimodel.core.event.ClickEvent;
import com.alphablox.blox.uimodel.core.event.IEventHandler;
import com.alphablox.blox.uimodel.core.event.ModelEvent;
import com.alphablox.internal.util.BloxConstants;
import com.alphablox.util.PropertyMap;

/**
 * Created by IntelliJ IDEA.
 * User: rfrankus
 * Date: Nov 5, 2003
 * Time: 4:36:27 PM
 * To change this template use Options | File Templates.
 */
public class TagGenerator {

    // Cell alert and cell format tags
    private static final String TAG_DESCRIPTION		= BloxConstants.DESCRIPTION_STRING;
    private static final String TAG_FOREGROUND		= BloxConstants.FORECOLOR_STRING;
    private static final String TAG_BACKGROUND		= BloxConstants.BACKCOLOR_STRING;
    private static final String TAG_CONDITION		= "condition";
    private static final String TAG_VALUE			= "value";
    private static final String TAG_VALUE2			= "value2";
    private static final String TAG_LINK			= "link";
    private static final String TAG_ALIGN			= BloxConstants.ALIGN_STRING;
    private static final String TAG_VALIGN			= BloxConstants.VALIGN_STRING;
    private static final String TAG_FORMAT			= BloxConstants.FORMAT_STRING;
    private static final String TAG_SCOPE			= "scope";
    private static final String TAG_FONT			= BloxConstants.FONT_STRING;
    private static final String TAG_APPLY			= "apply";
    private static final String TAG_IMAGE_ALIGN		= "image_align";
    private static final String TAG_IMAGE			= "image";
    private static final String TAG_GROUP           = "group";

    private String [] tags = { TAG_DESCRIPTION,
                               TAG_FOREGROUND,
                               TAG_BACKGROUND,
                               TAG_CONDITION,
                               TAG_VALUE,
                               TAG_VALUE2,
                               TAG_LINK,
                               TAG_ALIGN,
                               TAG_VALIGN,
                               TAG_FORMAT,
                               TAG_SCOPE,
                               TAG_FONT,
                               TAG_APPLY,
                               TAG_IMAGE,
                               TAG_IMAGE_ALIGN,
                               TAG_GROUP };

    private PresentBlox basePresentBlox;
    private PresentBlox presentBlox;
    private HttpServletRequest request;
    private HttpServletResponse response;

    private AppContext mAppContext;

    private Dialog dialog = new Dialog();
    private PresentBloxModel model;
    private HashSet ignoreParameters = new HashSet();
    // Stores parameters for whom null is a valid value.

    private static String indent = "    ";
    private static String[] bloxNames = {"Present","Grid","Chart","Page","Data","Toolbar","DataLayout"};

    public TagGenerator(PresentBlox presentBlox, AppContext appContext, HttpServletRequest request,HttpServletResponse response) throws InvalidParameterException,ServerBloxException,ModelException {
        this.presentBlox = presentBlox;
        this.request = request;
        this.response = response;
        this.mAppContext = appContext;
        init();
    }

    private void init() throws InvalidParameterException,ServerBloxException,ModelException {
        this.basePresentBlox = new PresentBlox();

        BloxContext context = BloxContextFactory.getBloxContext(request, response);
        basePresentBlox.init(context,"basePresentBlox");

        ignoreParameters.add("schema");
        ignoreParameters.add("catalog");
        ignoreParameters.add("userName");
        ignoreParameters.add("password");
        ignoreParameters.add("connectOnStartup");
        ignoreParameters.add("Data/visible");
        ignoreParameters.add("Present/visible");

        // deprecated properties (or not applicable to the DHTML client)
        ignoreParameters.add("border");

        model = presentBlox.getPresentBloxModel();
        addMenu();
    }

    private void addMenu() throws ModelException {
        Menubar menubar = model.getMainMenu();
        Menu actionMenu = (Menu) menubar.findInContainer("actionMenu");
        if(actionMenu==null) {
            actionMenu = new Menu("actionMenu", mAppContext.getString("TAGGEN_MENU_HEADER") );
            for(int i=0;i<menubar.size();i++) {
                Component component = menubar.get(i);
                if(component.getName().equals(ModelConstants.TOOLS_MENU)) {
                    menubar.insert(i+1,actionMenu);
                    break;
                }
            }
        }
        MenuItem tagGenMenuItem = new MenuItem("actionGeneratePresentBloxTag", mAppContext.getString("TAGGEN_MENU_LABEL") );
        actionMenu.add(tagGenMenuItem);
        model.getController().addEventHandler( new IEventHandler() {
            public boolean handleClickEvent( ClickEvent event ) throws Exception {
                if ( "actionGeneratePresentBloxTag".equals( event.getComponent().getName() ) ) {
                    String tagOutput = generateTag();
                    dialog.setWidth(620);
                    dialog.setHeight(500);
                    dialog.setModal(false);
                    dialog.setTitle( mAppContext.getString("TAGGEN_TEXTLABEL_TITLE") );
                    dialog.clear();
                    ComponentContainer componentContainer = new ComponentContainer();
                    componentContainer.setLayout(new VerticalLayout());
                    componentContainer.setAlignment(ComponentContainer.ALIGN_CENTER);

                    ComponentContainer container = new ComponentContainer();
                    container.setLayout(new HorizontalLayout());
                    container.setAlignment(ComponentContainer.ALIGN_CENTER);

                    Button close = new Button("dialogClose", mAppContext.getString("TAGGEN_BUTTON_CLOSE"));
                    close.setThemeClass("csCmpBg csThmClr csCmpBrdr csBtn csDfltBtnWdth");
                    container.add(close);

                    componentContainer.add(container);
                    componentContainer.add(new Spacer());

                    Edit edit = new Edit("bloxTagEdit");
                    edit.setLines(500);
                    edit.setHeight(400);
                    edit.setWidth(600);
                    edit.setValue(tagOutput);
                    edit.setSelected(true);
                    componentContainer.add(edit);

                    dialog.add(componentContainer);
                    dialog.setController(new DialogController(dialog) {
                        public boolean actionDialogClose(ModelEvent event) {
                            this.dialog.close();
                            return false;
                        }
                    });
                    model.getDispatcher().showDialog(dialog);
                    return true;
                }
                return false;
            }
        });
    }

    public String generateTag() throws ClassNotFoundException,GridBloxUnavailableException,ServerBloxMissingResourceException,ToolbarBloxUnavailableException,PageBloxUnavailableException,ChartBloxUnavailableException,DataLayoutBloxUnavailableException,ServerBloxException {
        // Blox objects for default PresentBlox and new one that relate to name of bloxNames array
        StringBuffer tagOutput = new StringBuffer();
        Object[] defaultBloxObject = {basePresentBlox,basePresentBlox.getGridBlox(),basePresentBlox.getChartBlox(),basePresentBlox.getPageBlox(),basePresentBlox.getDataBlox(),basePresentBlox.getToolbarBlox(),basePresentBlox.getDataLayoutBlox()};
        Object[] bloxObject = {presentBlox,presentBlox.getGridBlox(),presentBlox.getChartBlox(),presentBlox.getPageBlox(),presentBlox.getDataBlox(),presentBlox.getToolbarBlox(),presentBlox.getDataLayoutBlox()};

        if(presentBlox.getDataBlox().getDataSourceName()==null || "".equals(presentBlox.getDataBlox().getQuery())) {
            tagOutput.append( mAppContext.getString("TAGGEN_ERROR_NOQUERY" ) );
            return tagOutput.toString();
        }

        // Tag and Blox Class
        Class tagClazz;
        Class bloxClazz;
        // Empty argument to execute methods
        Object[] argObjs = new Object[ 0 ];
        // Create text area
        tagOutput.append("<blox:present");
        for (int bloxIdx = 0;bloxIdx <bloxNames.length;bloxIdx++) {
            // Get classes for each blox component
            tagClazz = Class.forName("com.alphablox.server.tags.simple.createTags."+bloxNames[bloxIdx]+"Tag");
            bloxClazz = Class.forName("com.alphablox.blox."+bloxNames[bloxIdx]+"Blox");

            // Get fields and methods for tag and blox class
            Field[] tagFields = tagClazz.getDeclaredFields();
            Method[] methods = bloxClazz.getMethods();

            // add <blox: for each component except present
            if(bloxIdx > 0)
                tagOutput.append(indent)
                    .append("<blox:")
                    .append(Character.toLowerCase(bloxNames[bloxIdx].charAt(0)))
                    .append(bloxNames[bloxIdx].substring(1));

            // For each tag field find the method
            for (int tagFieldIdx = 0; tagFieldIdx < tagFields.length; tagFieldIdx++) {
                Field field = tagFields[tagFieldIdx];
                String getFieldName = "get" + field.getName();
                String isFieldName = "is" + field.getName();

                //System.out.println("field = " + field);
                for (int j = 0; j < methods.length; j++) {
                    // only look at get and is methods
                    if(methods[j].getName().equalsIgnoreCase(getFieldName) || methods[j].getName().equalsIgnoreCase(isFieldName)) {
                        // make sure that no parameters are needed
                        if (methods[j].getParameterTypes().length==0) {
                            Object object1 = null;
                            Object object2 = null;
                            String value1 = null;
                            String value2 = null;
                            try {
                                // get values from each basePresentBlox and store them in an Object
                                object1 = methods[j].invoke(defaultBloxObject[bloxIdx],argObjs);
                                object2 = methods[j].invoke(bloxObject[bloxIdx],argObjs);
                                // Get the string value
                                value1 = String.valueOf(object1);
                                value2 = String.valueOf(object2);
                            } catch (IllegalAccessException e) {
                                System.out.println("IllegalAccess:"+e.getMessage());
                            } catch (IllegalArgumentException e) {
                                System.out.println("IllegalArgument:"+e.getMessage());
                            } catch (InvocationTargetException e) {
                                //System.out.println("InvocationTarget:"+e.getMessage()+":"+methods[j].getName()+":"+value2);
                            }

                            if (object2!=null) {
                                // Handle arrays and other types that are not automatically
                                // converted to String
                                if(object2 instanceof String[]) {
                                    value1 = "";
                                    value2 = "";
                                    String[] strings;
                                    if (object1!=null) {
                                        strings = (String[]) object1;
                                        for(int k=0;k<strings.length;k++) {
                                            if(k>0)
                                                value1+=",";
                                            value1+=strings[k];
                                        }
                                    }
                                    strings = (String[]) object2;
                                    for(int k=0;k<strings.length;k++) {
                                        if(k>0)
                                            value2+=",";
                                        value2+=strings[k];
                                    }
                                }
                                else if(object2 instanceof Dimension[]) {
                                    value1 = "";
                                    value2 = "";
                                    Dimension[] dimensions;
                                    if (object1!=null) {
                                        try {
                                            dimensions = (Dimension[]) object1;
                                            for (int k = 0; k < dimensions.length; k++) {
                                                Dimension dimension = dimensions[k];
                                                if(k>0)
                                                    value1+=",";
                                                value1+=dimension.getUniqueName();
                                            }
                                        } catch (Exception e) {
                                            System.out.println("Exception:"+e.getMessage());
                                        }
                                    }
                                    dimensions = (Dimension[]) object2;
                                    for (int k = 0; k < dimensions.length; k++) {
                                        try {
                                            Dimension dimension = dimensions[k];
                                            if(k>0)
                                                value2+=",";
                                            value2+=dimension.getUniqueName();
                                        } catch (Exception e) {
                                            System.out.println("Exception1:"+e.getMessage());
                                        }
                                    }
                                }
                                else if(object2 instanceof Rectangle) {
                                    try {
                                        Rectangle rectangle = (Rectangle) object2;
                                        value2 = rectangle.getX()+","+rectangle.getY()+","+rectangle.getWidth()+","+rectangle.getHeight();
                                    } catch (Exception e) {
                                        System.out.println("Exception2:"+e.getMessage());
                                    }
                                }

                                // Write parameters for all attributes that are different from default present
                                // also make sure that dataSourceName always gets written
                                if (!value2.equals(value1) || field.getName().equalsIgnoreCase("dataSourceName") || field.getName().equalsIgnoreCase("query")) {
                                    // Ignore parameters that are null
                                    // Handle bloxName because this is actually the id
                                    if(!value2.equalsIgnoreCase("null")) {
                                        try {
                                            if (field.getName().equalsIgnoreCase("bloxName")) {
                                                if (bloxIdx == 0) {
                                                    tagOutput.append("\n").append(indent);
                                                    tagOutput.append("id=\"").append(value2).append("\"");
                                                }
                                            }
                                            else if (field.getName().equalsIgnoreCase("query") && presentBlox.getDataBlox().generateQuery() != null) {
                                                tagOutput.append("\n").append(indent).append(indent);
                                                tagOutput.append("query=\"").append(replace(presentBlox.getDataBlox().generateQuery(), "\"", "\\\"")).append("\"");
                                            }
                                            else if (!ignoreParameter(bloxNames[bloxIdx], field.getName())) {
                                                tagOutput.append("\n").append(indent);
                                                if (bloxIdx > 0) {
                                                    tagOutput.append(indent);
                                                }
                                                tagOutput.append(field.getName()).append("=\"").append(replace(value2, "\"", "\\\"")).append("\"");
                                            }
                                        }
                                        catch (ServerBloxException e) {
                                            System.out.println("serverBloxException:" + e.getMessage());
                                        }
                                    }
                                }
                            }
                        }
                    }
                }
            }

            if(bloxIdx > 0) {
                if(bloxClazz.equals(GridBlox.class)) {
                    int[] cellAlertIds = presentBlox.getGridBlox().listCellAlertIds();
                    int[] cellEditorIds = presentBlox.getGridBlox().listCellEditorIds();
                    int[] cellFormatIds = presentBlox.getGridBlox().listCellFormatIds();
                    int[] cellLinkIds = presentBlox.getGridBlox().listCellLinkIds();
                    if(!(cellAlertIds.length==0 && cellEditorIds.length==0 && cellFormatIds.length==0 && cellLinkIds.length==0)) {
                        tagOutput.append(">\n");
                        for (int i = 0; i < cellAlertIds.length; i++) {
                            int cellAlertId = cellAlertIds[i];
                            tagOutput.append(indent).append(indent).append("<blox:cellAlert ");
                            tagOutput.append("index=\"").append(cellAlertId).append("\" ");
                            String value = presentBlox.getGridBlox().getCellAlert(cellAlertId);
                            PropertyMap alertProps = decode(value, tags);
                            for ( int j = 0; j < tags.length; j++)
                            {
                                String propertyValue = alertProps.get( tags[j] );
                                if ( propertyValue != null )
                                {
                                    tagOutput.append(tags[j]).append("=\"").append(propertyValue).append("\" ");
                                }
                            }
                            tagOutput.append("/>\n");
                        }
                        for (int i = 0; i < cellEditorIds.length; i++) {
                            int cellEditorId = cellEditorIds[i];
                            tagOutput.append(indent).append(indent).append("<blox:cellEditor ");
                            tagOutput.append("index=\"").append(cellEditorId).append("\" ");
                            String value = presentBlox.getGridBlox().getCellEditor(cellEditorId);
                            PropertyMap editorProps = decode(value, tags);
                            for ( int j = 0; j < tags.length; j++)
                            {
                                String propertyValue = editorProps.get( tags[j] );
                                if ( propertyValue != null ) {
                                    tagOutput.append(tags[j]).append("=\"").append(propertyValue).append("\" ");
                                }
                            }
                            tagOutput.append("/>\n");
                        }
                        for (int i = 0; i < cellFormatIds.length; i++) {
                            int cellFormatId = cellFormatIds[i];
                            tagOutput.append(indent).append(indent).append("<blox:cellFormat ");
                            tagOutput.append("index=\"").append(cellFormatId).append("\" ");
                            String value = presentBlox.getGridBlox().getCellFormat(cellFormatId);
                            PropertyMap formatProps = decode(value, tags);
                            for ( int j = 0; j < tags.length; j++)
                            {
                                // Skip the condition property.  It isn't supported in cellFormat.
                                if ( !tags[j].equals(TAG_CONDITION) )
                                {
                                    String propertyValue = formatProps.get( tags[j] );
                                    if ( propertyValue != null )
                                    {
                                        tagOutput.append(tags[j]).append("=\"").append(propertyValue).append("\" ");
                                    }
                                }
                            }
                            tagOutput.append("/>\n");
                        }
                        for (int i = 0; i < cellLinkIds.length; i++) {
                            int cellLinkId = cellLinkIds[i];
                            tagOutput.append(indent).append(indent).append("<blox:cellLink ");
                            tagOutput.append("index=\"").append(cellLinkId).append("\" ");
                            String value = presentBlox.getGridBlox().getCellLink(cellLinkId);
                            PropertyMap linkProps = decode(value, tags);
                            for ( int j = 0; j < tags.length; j++)
                            {
                                String propertyValue = linkProps.get( tags[j] );
                                if ( propertyValue != null )
                                {
                                    tagOutput.append(tags[j]).append("=\"").append(propertyValue).append("\" ");
                                }
                            }
                            tagOutput.append("/>\n");
                        }
                        tagOutput.append("</blox:grid>\n");
                    }
                    else {
                        tagOutput.append("/>\n");
                    }
                }
                else {
                    tagOutput.append("/>\n");
                }
            }
            else
                tagOutput.append(">\n");
        }
        tagOutput.append(indent);
        tagOutput.append("<bloxui:calculationEditor />\n");
        tagOutput.append("</blox:present>");
        return tagOutput.toString();
    }

    private boolean ignoreParameter(String bloxName, String fieldName) {
        return ignoreParameters.contains(fieldName) || ignoreParameters.contains(bloxName + "/" + fieldName);
    }


    private String replace(String string,String from,String to) {
        StringBuffer returnString = new StringBuffer();
        String str = string;
        while(str.indexOf(from)>-1) {
            int index = str.indexOf(from);
            returnString.append(str.substring(0,index));
            returnString.append(to);
            str = str.substring(index+from.length(),str.length());
        }
        returnString.append(str);
        return returnString.toString();
    }

    public static PropertyMap decode( String alertString, String [] tags )
	{
		if(alertString.length()==0)
			return null;
		PropertyMap values = new PropertyMap( );

		int startPos[] = new int[tags.length];
		int endPos[] = new int[tags.length];

		String lowerString = alertString.toLowerCase(); // convert to lower JUST to find indexes.

		// first let's get the positions of all the substrings
		for(int i = 0; i < tags.length; i++) {
			int lookFromPos = 0;
			boolean keepLooking = true;
			while(keepLooking) {
				startPos[i] = lowerString.indexOf(tags[i], lookFromPos);
				endPos[i] = startPos[i] + tags[i].length();
				if(startPos[i]>=0) {
					// we've found the string, now make sure '=' follows it
					String subString = lowerString.substring(endPos[i]).trim();
					char startChar = ( startPos[i] != 0 ) ? lowerString.charAt(startPos[i]-1) : ' ';
					if(subString.length()==0) {
						keepLooking = false;
					}
					else if(subString.charAt(0)!='=') {
						// this must be part of another string so ignore it
						lookFromPos = endPos[i];
					}
					else if ( !Character.isWhitespace(startChar) && startChar != ',' )
					{
						// This is not a tag match (probably ALIGN matching to VALIGN)
						lookFromPos = endPos[i];
					}
					else {
						// this is the real string so set position and stop looking
						while(Character.isWhitespace(lowerString.charAt(endPos[i])) ||
							lowerString.charAt(endPos[i])=='=')
							endPos[i]++;
						keepLooking = false;
					}
				}
				else {
					// the string was not found so ignore
					startPos[i] = -1;
					keepLooking = false;
				}
			}
		}

		// now we have the positions, let's get the values
		for(int i = 0; i < tags.length; i++) {
			if(startPos[i]>=0) {
				// search for the item after this one
				int nextPos = lowerString.length();
				for(int j = 0; j < tags.length; j++) {
					if(startPos[j] > endPos[i] && startPos[j] < nextPos) {
						nextPos = startPos[j];
					}
				}

				// now we have the substring, let's set it in the hashmap
				String value = alertString.substring(endPos[i], nextPos).trim();
				while(value.length()>0 && value.charAt(value.length()-1)==',')
					value = value.substring(0, value.length()-1).trim();

				// now get rid of quotes
				if(value.length()>0 && value.charAt(0)=='\"' &&
					value.charAt(value.length()-1)=='\"')
					value = value.substring(1, value.length()-1).trim();
				values.put(tags[i], value);
			}
		}
		return values;
	}
}
